#include "d_file.h"

D_File::D_File(char *filename, int mode,char *commentchar, int x1, int y1, int x2, int y2)
    : File(filename, mode, commentchar)
{
    char 
        *tmp_filename;
    if ((x2 - x1 - 7 <= 0) || (y2 - y1 - 5 <= 0))
        perror ("D_File: illegal viewport coordinates");

     //draw box around display
    box(x1, y1 + 1, x2, y2 - 1, BORDER_COLOR);
    //store coordinates of viewport
    col = x1 + 1;
    row = y1 + 2;
    width = x2 - x1 - 2;
    height = y2 - y1 - 3;

    //print filename, or if filename > widthjust print the first char
    // followed by ... and then the last char's
    if ( strlen(filename) > (unsigned)width)
    {
        tmp_filename = (char *)malloc(sizeof(char) * (width + 1));
        tmp_filename[0] = filename[0];
        tmp_filename[1] = '.';
        tmp_filename[2] = '.';
        tmp_filename[3] = '.';
        tmp_filename[width] = 0;
        strncpy(tmp_filename + 4, filename + (strlen(filename) - width + 4), width - 4);
    }
    else
    {       
        tmp_filename = strdup(filename);
    }
    print(x1 + 1, y1, FILENAME_COLOR, tmp_filename);

    //initialise position in file
    line_pos = 0;
    char_pos = 0;

    //initialise auxiliary buffers
    line_buffer = new char[width + 1];
    clear_line = new char[width + 1];
    line_buffer[width] = clear_line[width] = 0;
    for (int i = 0; i < width; i++)
        clear_line[i] = ' ';

    //initialise line_color array
    line_color = new int[get_number_of_lines() + 1];
    for (int i = 0; i <= get_number_of_lines(); i++)
    {
        line_color[i] = DIFF_LINE_COLOR;
    }
    vscroll = new Scrollbar(height, get_number_of_lines(), x2 - 1, y1 + 2, VERTICAL);
    update(x1, y1, x2, y2);
    refresh();
}

D_File::~D_File()
{
    delete [] line_buffer;
    delete [] clear_line;
    delete [] line_color;
    delete vscroll;
}

void D_File::set_color(int _line,int _color)
{
    if (_line > get_number_of_lines()) 
        _line = get_number_of_lines();
    line_color[_line] = _color;
}


int D_File::jump_to(int _line)
{
    if (_line >= get_number_of_lines())
        _line = get_number_of_lines();
    if (_line < 0) 
        _line = 0 ;
    line_pos = _line;
    refresh();
    return _line;
}

int D_File::scroll_lines(int lines)
{
    int l;
    if (lines < 0)
    {
        for (l = -1; l >= lines; l--)
        {
            if (line_pos  <= 0)
                break;
            line_pos--;
            refresh();
        }
    }
    else
    {
        for (l = 1; l <= lines; l++)
        {
            if (line_pos >= get_number_of_lines())
                break;
            line_pos++;
            refresh();
        }
    }
    return line_pos;

}

void D_File::jump_to_char(int _char)
{
    if (_char<0) 
        _char = 0;
    char_pos = _char;
    refresh();
}

void D_File::jump_to_eol()
{
        char_pos = strlen(line(line_pos)) - width + 1;
    if (char_pos <0)
        char_pos = 0;
    refresh();
}

int D_File::scroll_chars(int characters)
{
    int l;
    if (characters < 0)
    {
        for (l = -1; l >= characters; l--)
        {
            if (char_pos - 1 < 0)
                break;
            char_pos--;
            refresh();
        }
        return ++l;
    }
    else
    {
        for (l = 1; l <= characters; l++)
        {
            char_pos++;
            refresh();
        }
        return --l;
    }

}

int D_File::scroll_to(int _line)
{
    if (_line < 0) 
        _line = 0;
    if (_line > get_number_of_lines()) 
        _line = get_number_of_lines();
    int linediff = _line -line_pos;
    scroll_lines(linediff);
    return _line;
}

void D_File::refresh()
{
    int color,copylength;
    for (int y = -(height / 2);y <= (height / 2);  y++)
    {
        int lpos = line_pos + y;
        if (lpos > get_number_of_lines())
            lpos = get_number_of_lines();
        if (lpos >=0)
            color = line_color[lpos];
        else
            color = 0;

        if (!y) color += 16;

        strcpy (line_buffer, clear_line);

        
        if (lpos >= 0)
        {
            if (char_pos < (int)strlen(line(lpos)))
            {
                copylength = strlen(line(lpos) + char_pos);
                copylength = (copylength > width) ? width : copylength;
                strncpy(line_buffer, line(lpos) + char_pos, copylength);
            }
        }
        
//        print (col, row + y + height /2, color, clear_line);
        print (col, row + y + height /2, color, line_buffer);
        update_keyboard_buffer();
                
    }
    vscroll->shift_to(line_pos);
    sprintf(line_buffer,"%d %d ",line_pos + 1,char_pos + 1);
    print(col, row + height + 1, LINE_NUMBER_COLOR,line_buffer);
    update(col,row,col + width, row + height-1);
    update(col,row + height + 1,col + width, row + height + 1);
}

int D_File::curline()
{
    return line_pos;
}

int D_File::dis_height()
{
    return (height);
}


void D_File::insert_line(int number,char const *line)
{
        line_color = (int *)realloc(line_color,(get_number_of_lines()+2) * sizeof (int));
        for (int i=get_number_of_lines()+1;i>number;i--)
        {
            line_color[i] = line_color[i - 1];    
        }
        line_color[number] = 7;
        File::insert_line(number,line);
        vscroll->reset_real_length(get_number_of_lines());
}

void D_File::delete_line(int number)
{
        if (number == get_number_of_lines())
            return;
        for (int i=number;i<=get_number_of_lines();i++)
        {
            line_color[i] = line_color[i + 1];    
        }

        File::delete_line(number);
        line_color = (int *)realloc(line_color,(get_number_of_lines()+1) * sizeof (int));
        if (get_number_of_lines())
            vscroll->reset_real_length(get_number_of_lines());

}
