#ifndef _file_h_
#define _file_h_

//defines for the spacefiltering mode when reading the file 
#define MODE_LITERAL 0
#define MODE_CODE    1
#define MODE_REMOVE_ALL_SPACES   2
#define MODE_IGNORE_COMMENT      4


#define EXPAND_TABS  1
#define TABSIZE 4
#define TAB 9

#define INPUT_BUFFERSIZE 1024

#include <malloc.h>
#include <strings.h>
#include "perror.h"
#include "macros.h"

class File : public Perror
{
    public:
    //constructors
    File(void);
    File(char *filename, int mode,char *commentchar);

    //copy constructor
    File(File const &f);

    //overloaded assignment
    File const &operator= (File const &other);
    //destructor
    ~File();

    //search for the nearest occurence of string(nearest to line number)
    int search(int number, const char *string) const;

    //retourneer Line #number
    const char *line(int number) const;

    int get_number_of_lines() const;
    void set_tag(int line, int tag);
    int get_tag(int line) const;

    //this determines the behaviour of the search routine
    // search will return -1 for any string with length length or shorter
    // entering -1 will cause search to search for all strings
    //default is 0 (only empty lines will not be matched)
    void s_length(int _length);

    // this inserts a line above line number
    void insert_line(int number, char const *line);

    void delete_line(int number);

    private:

    int mode;
    int search_length;

    //line tag, in this int you can store info on the line, they are initialised to -1
    int *line_tag;
    char **Line;
    char *comment;
    int number_of_lines;
    char *remove_spaces(char *line);
    void expand_tabs(char *line);

    //compare function,compares Line[number] width string, returns 1 if they
    // compare, 0 if they differ or if string is only white space
    int compare(int number, const char *string) const;

    //primitive functions used in copy constructor,overloaded=, and destructor
    void copy(File const &other);
    void destroy();

    void remove_eol_comment(char *string) const;
};

#endif
