#include "main.h"

#ifdef GRAPHICAL
#include "math.h"
#include <time.h>
#endif

#define MAJOR_VERSION 0 
#define MINOR_VERSION 9
#define MINOR_MINOR_VERSION 7
#define DATE  "12 feb. 97"

void usage();
void init_colors_tags();
void enable_scroll();
void scroll(int direction);
void options_filter(int &argc, char **argv);
void searchdiff(int dir);
void writefile(File *file,char *name);

static D_File  
	*file1,
	*file2;


//global ints to turn on/off options
	int MODE = MODE_CODE;
	int SEARCH_LENGTH=0;
	int SCROLL_BEHAVIOUR = SCROLL;
	int SCREEN_LENGTH = 25;
	
	char *commentchar;

int isCHANGED = 0;
int VIEWER_MODE = 0;
	Keyboard 
		*keyboard;


int main(int argc, char **argv)
{
	keyboard = new Keyboard;
	#ifdef GRAPHICAL
		SCREEN_LENGTH = 50;
	#endif
	options_filter(argc, argv);
	if (argc != 3 && argc !=2)
		usage();
	//load files
	if (argc == 2)
		VIEWER_MODE = 1; //use the program as viewer
	
	if (!VIEWER_MODE)
	{
		file1 = new D_File(argv[1], MODE, commentchar, 1, 1, 40, SCREEN_LENGTH);
		file2 = new D_File(argv[2], MODE, commentchar, 41, 1, 80, SCREEN_LENGTH);
	}
	else
	{
		file1 = new D_File(argv[1], MODE, commentchar, 1, 1, 80, SCREEN_LENGTH);
	}

	if (!VIEWER_MODE) 
	{
		file2->s_length(SEARCH_LENGTH);
		//initialise from differences between files
		init_colors_tags();
	}
	
	//do what it's for
	enable_scroll();
	if (isCHANGED)
		writefile(file1, "outfile");
	//cleanup
	delete file1;
	if (!VIEWER_MODE) delete file2;


	printf(

		  "Thanks for using this program. If you think it's usefull, please spread it to\n" 
		  "all your friends.\n"
		  "Send any comments to:\n" 
		  "m.versteegh@cpedu.rug.nl\n" 
		  "or\n" 
		  "Martijn Versteegh, postbus 1443, 9701BK Groningen, The Netherlands\n\n"
		  "This program was written using the wonderful DJGPP 32-bits C/C++ compiler.\n"
		  "DJGPP can be obtained for free!!!! see http://www.delorie.com for more details.\n"
		  );

	#ifdef GRAPHICAL
		printf("\nThis graphical version uses the Allegro game programming library for DJGPP, by\n" 
			   "Shawn Hargreaves, for it's screen output, this great library can be obtained\n" 
			   "from the same places as DJGPP.\n");
	#endif

	printf("\nRun the program without any command-line options to get help\n");
	if (isCHANGED)
	{
		int formerattrib = ScreenAttrib;
		textattr(4);
		cputs("\n\nwrote left panel to file OUTFILE\n");    
		textattr(formerattrib);
	}
}

void enable_scroll()
{
	int key;
	while (1)
	{
		key = keyboard->wait_key_pressed();
		if (key == ESCAPE) 
			break;
		if (key == ARROW_UP)
			scroll(-1);
		if (key == CTRL_ARROW_UP)
			file2->scroll_lines(-1);
		if (key == CTRL_ARROW_DOWN)
			file2->scroll_lines(1);
		if (key == ARROW_DOWN)
			scroll(1);
		if (key == HOME)
		{
			file1->jump_to_char(0);
			if (!VIEWER_MODE)
				file2->jump_to_char(0);
		}
		if (key == END)
		{
			file1->jump_to_eol();
			if (!VIEWER_MODE)
				file2->jump_to_eol();
		}
		if (key == DEL && !VIEWER_MODE)
		{
			if (file1->get_tag(file1->curline()) >=0)
			{
				file2->set_tag(file1->get_tag(file1->curline()),-1);
				file2->set_color(file1->get_tag(file1->curline()),DIFF_LINE_COLOR);
			}
			file1->delete_line(file1->curline());
			for (int i=0;i<file2->get_number_of_lines();i++)
			{
				if (file2->get_tag(i) >file1->curline())
					file2->set_tag(i,file2->get_tag(i) - 1);
			}

			//get the line file->2 should scroll to

			
			int
				to_line2 = file1->get_tag(file1->curline());

			//and do it(if there is one)
			if (to_line2 >= 0)
				file2->jump_to(to_line2);



			file1->refresh();
			file2->refresh();
			isCHANGED = 1;
		}
		
		if (key == INS && !VIEWER_MODE)
		{
			file1->insert_line(file1->curline(),file2->line(file2->curline()));
			file1->set_tag(file1->curline(),file2->curline());
			file1->set_color(file1->curline(),SAME_LINE_COLOR);
			for (int i=0;i<file2->get_number_of_lines();i++)
			{
				if (file2->get_tag(i) >= file1->curline())
					file2->set_tag(i,file2->get_tag(i) + 1);
			}
			file2->set_tag(file2->curline(),file1->curline());
			file2->set_color(file2->curline(),SAME_LINE_COLOR);
			file2->scroll_lines(-1);
			file1->refresh();
			file2->refresh();

			isCHANGED = 1;
		}
		
		if (key == ARROW_LEFT)
		{
			file1->scroll_chars(-1);
			if (!VIEWER_MODE) file2->scroll_chars(-1);
		}
		if (key == ARROW_RIGHT)
		{
			file1->scroll_chars(1);
			if (!VIEWER_MODE) file2->scroll_chars(1);
		}
		if (key == PG_UP)
			scroll(-10);
		if (key == PG_DOWN)
			scroll(10);
		if (key == NEXTDIFF && !VIEWER_MODE)        
			searchdiff(1);
		if (key == LASTDIFF && !VIEWER_MODE)        
			searchdiff(-1);

	}

}

void scroll(int lines)
{
	int to_line2;
	int tmp_line;
	if (SCROLL_BEHAVIOUR == SCROLL)
	{
		//scroll file1
		file1->scroll_lines(lines);
		if (VIEWER_MODE) return;
		//get the line that file2 should scroll to
		to_line2 = file1->get_tag(file1->curline());

		if (to_line2 < 0) 
		{
			tmp_line = file1->curline();
			while(--tmp_line >= 0 && (to_line2 = file1->get_tag(tmp_line)) < 0);
			to_line2++; //1 line onder de last matching line
		}
		file2->scroll_to(to_line2);
	}
	else
	{
		//scroll file1
		file1->jump_to(file1->curline() + lines);
		if (VIEWER_MODE) return;

		//get the line that file2 should scroll to
		to_line2 = file1->get_tag(file1->curline());

		//and do it(if there is one)
		if (to_line2 >= 0)
			file2->jump_to(to_line2);
	}

}

void usage()
{
	fprintf(stderr, 
			"Version %d.%d.%d  date:%s\n"
			"Usage:\n"
			"CMP <file1> <file2> [-a | -#][-l][-j][-s][-c]\n"
			"CMP <file> [-l][-j]\n"
			"-l:   compare literal, do not compress spaces\n"
			"-a:   try to match everything, even empty lines\n"
			"-#:   do not try to match lines with only # or less characters (# out of 0-9)\n"
			"-j:   jump rather than scroll\n"
			"-s:   ignore ALL white space when comparing\n"
			"-c[comment-string]:   ignore with comment-string outcommented code when\n" 
			"    comparing. so -c# ignores everything after a #, '-c' is the same as '-c//'\n"
			"\nWhen used width only one file, cmp will act as a viewer.\n"
			"\n"
			" In the program these keys do something:\n"
			"ESC quits.\n"
			"ARROWS,PGUP,PGDN,HOME,END move the files.\n"
			"+,-(on the numpad) jump to the next/last difference.\n"
			"CTRL-ARROWUP/CTRL-ARROWDOWN moves only the second file.\n"
			"INS copies the current line from panel 2 into panel 1.\n"
			"DEL deletes the current line from panel 1.\n"
			"When INS/DEL is used, panel 1 is written to a file called OUTFILE on exit.\n"
			,MAJOR_VERSION,MINOR_VERSION,MINOR_MINOR_VERSION,DATE
			);    
	exit(1);
}

void init_colors_tags()
{
	int counter2 = 0;
	int tmp_counter;
	//laat de 2 laatste regels naar elkaar wijzen
	file1->set_tag(file1->get_number_of_lines(), file2->get_number_of_lines());
	file2->set_tag(file2->get_number_of_lines(), file1->get_number_of_lines());
	for (int counter1 = 0; counter1 < file1->get_number_of_lines(); counter1++)
	{
		tmp_counter = file2->search(counter2, file1->line(counter1));
		if (tmp_counter >= 0)
		{
			counter2 = tmp_counter;
			file1->set_tag(counter1, counter2);
			file1->set_color(counter1, SAME_LINE_COLOR);
			file2->set_tag(counter2, counter1);
			file2->set_color(counter2, SAME_LINE_COLOR);
		}
	}
	file1->refresh();
	file2->refresh();
}

void turn_on_option(char *option)
{
	if (isdigit(option[0]))
	{
		SEARCH_LENGTH = option[0] - '0';
		return;
	}
	switch(option[0])
	{
		default:
			fprintf(stderr, "unkown option : -%s\n",option);
			exit(1);
		case 'l':
			MODE = MODE_LITERAL;
			break;
		case 's':
			MODE =(MODE | MODE_REMOVE_ALL_SPACES);
			break;    
		case 'c':
			MODE = (MODE | MODE_IGNORE_COMMENT);
			if (strlen(option)>1)
			{
				commentchar = strdup(option + 1);
			}
			else
			{
				commentchar = "//";
			}
			break;    

		case 'a':
			SEARCH_LENGTH = -1;
			break;
		case 'j':
			SCROLL_BEHAVIOUR = JUMP;
			break;
	}
}

void options_filter(int &argc, char **argv)
{
	int 
		index = 0;
	while(argv[index])
	{
		if (argv[index][0] == '-')
		{
			turn_on_option(argv[index] + 1);
			for (int i = index; i < argc ; i++)
			{
				argv[i] = argv[i + 1];
			}
			argc--;
		}
		else
			index++;
	}
}
//search the next difference in direction dir
void searchdiff(int dir)
{
	int line = file1->curline();
	int tag;
	line += dir;
	tag = file1->get_tag(line);
	while (tag >= 0 && tag == file1->get_tag(line - dir) + dir)
	{                                       
		line += dir;
		tag = file1->get_tag(line);
		if (line < 0 && line >= file1->get_number_of_lines())
			break;
	}
	int scroll_amount = line - file1->curline();
	if (!scroll_amount) scroll_amount = dir; //at least scroll 1 step
	scroll(scroll_amount);
	
}


void writefile(File* file,char *name)
{
	FILE *outfile;
	if (!(outfile =fopen(name,"w")))
		perror("unable to open outputfile");
		
	char line_buffer[INPUT_BUFFERSIZE+2];
	for (int i=0;i<file->get_number_of_lines();i++)
	{
		strcpy(line_buffer,file->line(i));
		strcat(line_buffer,"\n");
		if (fputs(line_buffer,outfile) == EOF)
			perror("unable to write to outputfile");

	}
	fclose(outfile);
}
/*

void screensaver()
{
#ifdef GRAPHICAL
	BITMAP *buffer = create_bitmap(640,400);
	BITMAP *buffer2 = create_bitmap(640,400);
	blit(screen,buffer,0,0,0,0,640,480);
	double rot=0;
	int time;
	while(!kbhit())
	{
		rot = rot +0.01;
		if (rot>2*6.283)
			rot=0;
		time = clock();
		while(clock()<time+1);
		clear(buffer2);
		blit(buffer,buffer2,0,0,(int)(160*sin(rot)), (int)(100 *sin(rot)),640,480);
		vsync();
		blit(buffer2,screen,0,0,0,0,640,480);

	}
	blit(buffer,screen,0,0,0,0,640,480);
	destroy_bitmap(buffer);
	destroy_bitmap(buffer2);
#endif
}
  */
